
(exit "$?") &&
prj_unset spf_host spf_base spf_btag spf_version spf_vtag &&

spf_parse() {
  sp_parse "${1?}" &&
  case ${sp_category?} in
    host/*/*/foreign/) return 100;;
    host/*/foreign/) :;;
    *) return 100;;
  esac &&
  case ${sp_base?} in
    ?*+spf+*)
      spf_base=`prj_sedarg1 's/+spf+.*//' "${sp_base?}"` &&
      spf_btag='
/+spf+/!d
:trim
s:^.[^+]*+:+:
/^+spf+/!btrim
' &&
      spf_btag=`prj_sedarg1 "${spf_btag?}" "${sp_version?}"`;;
    *) spf_base=${sp_base?} && spf_btag=;;
  esac &&
  spf_host='s:^host/\(.*\)/foreign/$:\1:' &&
  spf_host=`prj_sedarg1 "${spf_host?}" "${sp_category?}"` &&
  if test '' = "${sp_version?}"; then spf_version= && spf_vtag=
  else
    spf_version='s:^-::
s:+spf+.*$::' &&
    spf_version=`prj_sedarg1 "${spf_version?}" "${sp_version?}"` &&
    spf_vtag='
/+spf+/!d
:trim
s:^.[^+]*+:+:
/^+spf+/!btrim
' &&
    spf_vtag=`prj_sedarg1 "${spf_vtag?}" "${sp_version?}"`
  fi &&
  spf_path_snippet=${sp_path?}/package/foreign_info
} &&

spf_load_info() {
  : "${sp_path?}" "${spf_version?}" &&
  prj_x2 prj_set SPF_INFO "${1?}" &&
  spf_srcdir_=${spf_base?}-${spf_version?} &&
  spf_url_home_= &&
  spf_url_watch_= &&
  spf_url_srcs_= &&
  spf_tested_versions= &&
  spf_ccvars_= &&
  prj_u2 prj_set spf_path_prefix  "${sp_path?}"/prefix &&
  prj_u2 prj_set spf_path_cc      "${sp_path?}"/conf-compile &&
  prj_u2 prj_set spf_path_compile "${sp_path?}"/compile &&
  prj_u2 prj_set spf_path_conf    "${sp_path?}"/conf &&
  prj_x2 prj_set SPF_CONF         "${spf_path_conf?}" &&
  prj_u2 prj_set spf_path_data    "${SPF_CONF?}/${spf_base?}-data"/prefix &&
  prj_x2 prj_set_default SPF_LIBC glibc &&
  prj_u2 prj_set spf_path_libc    "${spf_path_conf?}/${SPF_LIBC?}" &&
  case $SPF_INFO in
    */*) . "${SPF_INFO?}";;
    *) . ./"${SPF_INFO?}";;
  esac &&
  prj_u2 prj_set spf_path_src     "${spf_path_compile?}"/src/"${spf_srcdir_?}"
} &&

prj_unset spf_srcdir_ &&
spf_srcdir() {
  spf_srcdir_=${1?} &&
  prj_u2 prj_set spf_path_src "${sp_path?}"/compile/src/"${spf_srcdir_?}"
}

prj_unset spf_url_home_ spf_url_watch_ spf_url_srcs_ &&
spf_url() { : "${1?}" && spf_url_"$@"; } &&
spf_url_home()  { : "${1?}" && spf_url_home_=${1?}; } &&
spf_url_watch() { : "${1?}" && spf_url_watch_=${1?}; } &&
spf_url_src_add() {
  prj_qlist_push spf_url_srcs_ "${1?}" &&
  prj_qlist_push spf_url_srcs_ "${2-.}"
} &&
spf_url_src() {
  spf_url_srcs_= &&
  prj_qlist_push spf_url_srcs_ "${1?}" &&
  prj_qlist_push spf_url_srcs_ "${2-.}"
} &&

prj_unset spf_tested_versions &&
spf_tested_version() { prj_qlist_push spf_tested_versions "${1?}"; } &&

prj_u2 prj_set spf_cmd_list_noop '' &&
prj_u2 prj_set spf_cmd_list_hide '' &&
spf_cmd() {
  prj_qlist_push spf_cmd_list_"$1" "${2?}"
} &&

spf_hack() { spf_hack_${1+"$@"}; } &&
spf_hack_errno() {
  prj_sedfile "$1" 's:^extern int errno;$:#include <errno.h>:'
} &&

#spf_dep() { :; } && 

spf_rc() { spf_rc_${1+"$@"}; } &&

spf_is_token() {
  case ${1?} in
    ''|\
    [!ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz]*|\
    *[!ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz_0123456789]*)
      return 100;;
    *) return 0;;
  esac
} &&
spf_need_token() {
  : "${1?}" &&
  { spf_is_token "$1" || prj_fail "not a token: $1"; }
} &&

spf_xesc() {
  prj_x2 prj_capture "${1?}" prj_sedarg1 "s/[$3\\\\&]/\\\\&/g" "${2?}"
} &&

prj_u2 prj_set spf_links '' &&
spf_link() {
  spf_need_token "${2?}" &&
  { test + = "$1" || test = = "$1" || prj_fail "not '+' or '=': $1"; } &&
  prj_append spf_links "$2" ' ' &&
  { test + = "$1" || prj_set spf_links_"$2" ''; } &&
  eval '
    shift && shift &&
    while test "$#" != 0; do
      { prj_qlist_push spf_links_'"$2"' "$1" &&
        shift
      } || return "$?"
    done'
} &&
spf_link = command bin sbin &&
spf_link = include include &&
spf_link = library lib &&
spf_link = man man share/man &&

spf_data() { : "${3?}" && spf_data_${1+"$@"}; } &&
spf_data_file()  { spf_data_ f "$1" "${2?}"; } &&
spf_data_dir()   { spf_data_ d "$1" "${2?}"; } &&
spf_data_other() { spf_data_ e "$1" "${2?}"; } &&
prj_u2 prj_set spf_data_list '' &&
spf_data_() {
  : "${3?}" &&
  prj_qlist_push spf_data_list "$1" &&
  prj_qlist_push spf_data_list "$2" &&
  prj_qlist_push spf_data_list "$3"
} &&

prj_unset spf_ccvars_ &&
spf_cc() {
  : "${3?}" &&
  spf_need_token "${1?}" &&
  prj_qlist_push spf_ccvars_ "$1"
} &&

spf_ccload() {
  eval "set x ${spf_ccvars_?}" && shift &&
  prj_unset spf_ccvar spf_ccdir &&
  spf_ccdir=${sp_path?}/conf-compile/ &&
  for spf_ccvar in ${1+"$@"}; do
    { if test -f "${spf_ccdir?}${spf_ccvar?}"
        then prj_u2 prj_capture spf_ccv_"${spf_ccvar?}" \
          sed q "${spf_ccdir?}${spf_ccvar?}"
        else prj_u2 prj_capture spf_ccv_"${spf_ccvar?}" \
          sed q "${spf_ccdir?}defaults/${spf_ccvar?}"
      fi
    } || exit "$?"
  done
} &&

spf_cc_() { : "${1?}" && spf_cc_"$@"; } &&

spf_cc_cppflags() {
  spf_cc cppflags "$1" 'These arguments are passed to the C preprocessor.'
} &&

spf_cc_cxxflags() {
  spf_cc cxxflags "$1" 'These arguments are passed to the C++ compiler.'
} &&

spf_cc_cflags() {
  spf_cc cflags "$1" 'These arguments are passed to the C compiler.'
} &&

spf_cc_ldflags() {
  spf_cc ldflags "$1" 'These arguments are passed to the linker.'
} &&

spf_cc_nls() {
  spf_cc nls y \
    'This specifies whether to build the package with locale support.'
} &&

spf_cc_configure_args() {
  spf_cc configure_args "$1" 'These arguments are passed to ./configure.'
} &&

spf_cc_make_args() {
  spf_cc make_args "$1" 'These arguments are passed to make.'
} &&

prj_u2 prj_set spf_c_nl '
' &&
prj_u2 prj_set spf_c_tab '	' &&
prj_u2 prj_set spf_c_ws   " ${spf_c_tab?}" &&
prj_u2 prj_set spf_c_cws  "[${spf_c_ws?}]" &&
prj_u2 prj_set spf_c_cwsp "${spf_c_cws?}${spf_c_cws?}*" &&

spf_edit() { spf_edit_${1+"$@"}; } &&
spf_edit_1c() {
  prj_sedfile "$1" "1c\\${spf_c_nl?}${2?}\\${spf_c_nl?}"
} &&
prj_unset spf_edit_c_define_s &&
spf_edit_c_define() {
  : "${4?}" &&
  if test string = "$1"
    then spf_edit_c_define_s=`prj_quote_c "$4"`
    else spf_edit_c_define_s=$4
  fi &&
  spf_edit_c_define_s=`prj_quote_sed : "${spf_edit_c_define_s?}"` &&
sha1sum "$2" &&
set -xv &&
  prj_sedfile "$2" "s:^\\(${spf_c_cws?}*#${spf_c_cws?}*define${spf_c_cwsp?}$3${spf_c_cwsp?}\\)[^${spf_c_ws?}].*$:\\1${spf_edit_c_define_s?}:" &&
set +xv &&
sha1sum "$2"
} &&

spf_once() { "$@"; } && 

#spf_warn() {
#  echo_ >&2 "${spf_program?}: warning: $*" &&
#  sleep 2 &&
#  echo_ >&2 "${spf_program?}: continuing"
#} &&

spf_do_install() { make all install; } &&
spf_do_check() { make check; } &&

spf_style() { : "${1?}" && spf_style_"$@"; } &&

spf_style_gnu() {
  spf_url home  "http://www.gnu.org/software/${spf_base?}/" &&
  spf_url watch "http://ftp.gnu.org/gnu/${spf_base?}/" &&
  spf_url src   "http://ftp.gnu.org/gnu/${spf_base?}/${spf_base?}-${spf_version?}.tar.bz2" &&
  spf_cc_ cppflags &&
  spf_cc_ cxxflags &&
  spf_cc_ cflags &&
  spf_cc_ ldflags &&
  spf_cc_ configure_args &&
  spf_cc_ make_args &&
  prj_set spf_style_gnu_dir_ . &&
  spf_style_gnu_dir() { spf_style_gnu_dir_=${1?}; } &&
  prj_set spf_style_gnu_configure_args_ '' &&
  spf_style_gnu_configure_arg() {
    prj_qlist_push spf_style_gnu_configure_args_ "${1?}"
  } &&
  spf_style_gnu_do_before_configure() { :; } &&
  spf_style_gnu_do_configure() {
    prj_getstatus spf_style_gnu_had_libc test -d "${spf_path_libc?}"/. &&
    prj_getstatus spf_style_gnu_had_gcc  test -d "${spf_path_conf?}"/gcc/. &&
    set SPF_GCC_GCCLIB=y RANDOM="$$" "${spf_style_gnu_dir_?}"/configure &&
    eval 'env "$@" ' \
      "${spf_style_gnu_configure_args_?} ${spf_ccv_configure_args?}" &&
    { test "${spf_style_gnu_had_libc?}" = 0 || rm -f "${spf_path_libc?}"; } &&
    { test "${spf_style_gnu_had_gcc?}"  = 0 || rm -f "${spf_path_conf?}"/gcc; }
  } &&
  spf_style_gnu_do_before_make()    { :;                                 } &&
  spf_style_gnu_do_make()           { eval "make ${spf_ccv_make_args?}"; } &&
  spf_style_gnu_do_before_install() { :;                                 } &&
  spf_style_gnu_do_install()        { make install;                      } &&
  spf_style_gnu_do_before_check()   { :;                                 } &&
  spf_style_gnu_do_check()          { make check;                        } &&
  spf_style_gnu_do_after_check()    { :;                                 } &&
  spf_do_install() {
    spf_style_gnu_configure_arg --prefix="${spf_path_prefix?}" &&
    if test n = "$spf_ccv_nls"; then
      spf_style_gnu_configure_arg --disable-nls
    elif prj_is_set spf_ccv_nls; then
      spf_style_gnu_configure_arg --enable-nls
    else :
    fi &&
    if prj_is_set spf_ccv_dep_xorg &&
       test -d "${spf_ccv_dep_xorg?}"; then
      spf_style_gnu_configure_arg \
        --x-includes="${spf_path_conf?}"/xorg/include &&
      spf_style_gnu_configure_arg \
        --x-libraries="${spf_path_conf?}"/xorg/library
    else :
    fi &&
    prj_x2 prj_set CPPFLAGS "${spf_ccv_cppflags?}" &&
    prj_x2 prj_set CXXFLAGS "${spf_ccv_cxxflags?}" &&
    prj_x2 prj_set CFLAGS   "${spf_ccv_cflags?}" &&
    prj_x2 prj_set LDFLAGS  "${spf_ccv_ldflags?}" &&
    #prj_x2 prj_set CPP    'gcc -E' && 
    #prj_x2 prj_set CXXCPP 'g++ -E' && 
    spf_once spf_style_gnu_do_before_configure &&
    spf_once spf_style_gnu_do_configure        &&
    spf_once spf_style_gnu_do_before_make      &&
    spf_once spf_style_gnu_do_make             &&
    spf_once spf_style_gnu_do_before_install   &&
    spf_once spf_style_gnu_do_install          &&
    spf_once spf_style_gnu_do_before_check
  } &&
  spf_do_check() {
    spf_style_gnu_do_before_check &&
    spf_style_gnu_do_check &&
    spf_style_gnu_do_after_check
  }
} &&

spf_style_djb() {
  spf_url home  "http://cr.yp.to/${spf_base?}.html" &&
  spf_url watch "http://cr.yp.to/${spf_base?}/install.html" &&
  spf_url src   "http://cr.yp.to/${spf_base?}/${spf_base?}-${spf_version?}.tar.gz" &&
  spf_cc_ cppflags &&
  spf_cc_ cflags &&
  spf_cc_ ldflags &&
  spf_cc_ make_args &&
  spf_style_djb_do_before_install() { :; } &&
  spf_style_djb_do_after_install() { :; } &&
  spf_do_install() {
    spf_style_djb_do_before_install &&
    spf_edit_1c conf-home "${spf_path_prefix?}" &&
    spf_edit_1c conf-cc \
      "gcc -O2 ${spf_ccv_cppflags?} ${spf_ccv_cflags?}" &&
    spf_edit_1c conf-ld \
      "gcc -s ${spf_ccv_cppflags?} ${spf_ccv_cflags?} ${spf_ccv_ldflags?}" &&
    if test -f error.h; then spf_hack_errno error.h; else :; fi &&
    eval "make ${spf_ccv_make_args?}" &&
    make setup check &&
    spf_style_djb_do_after_install
  } &&
  spf_do_check() { :; }
} &&

spf_style_python() {
  : 
} &&

spf_style_perl() {
  : 
}
